import React, { Component } from 'react';
import keycode from 'keycode';
import { addIndex, map, pick } from 'ramda';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';

import {
  KeystrokeHistory, 
  keystrokeHistoryActions,
} from '../../src/KeystrokeHistory';

const mapIndexed = addIndex(map);

const mapStateToProps = state => ({ state });

const mapDispatchToProps = dispatch => ({
  actions: bindActionCreators(keystrokeHistoryActions, dispatch),
});

class HistoryTable extends Component {
  render() {
    const { history, startTime } = this.props;

    return (
      <table>
        <thead>
          <tr><th>key</th><th>down</th><th>up</th></tr>
        </thead>
        <tbody>
          {
            mapIndexed(
              ({ keycode: code, down, up }, index) => (
                <tr key={ index }>
                  <td>{ keycode(code) }</td>
                  <td>{ down - startTime }</td>
                  <td>{ (up - startTime) || ''  }</td>
                </tr>
              ),
              history,
            )
          }
        </tbody>
      </table>
    )
  }
}

@connect(mapStateToProps, mapDispatchToProps)
export default class App extends Component {
  constructor(props) {
    super(props);

    this.startTime = new Date().getTime();
  }

  onButtonClick = () => {
    this.props.actions.clearValue();
  };

  onInputChange = (e) => {
    if (e.target.value === 'clear') {
      this.props.actions.clearValue();
    }
  };

  render() {
    const { history, value } = this.props.state;

    const abbrevHistory = history.slice(Math.max(0, history.length - 15));

    return (
      <div>
        <KeystrokeHistory
          className="green"
          onInputChange={ this.onInputChange }
          state={ this.props.state }
        />
        <div> input value: { value } </div>
        <div>
          <button onClick={ this.onButtonClick }> clear input </button>
        </div>
        <HistoryTable
          history={ abbrevHistory }
          startTime={ this.startTime }
        />
      </div>
    );
  }
}
